from cms.api import add_plugin
from cms.models import Placeholder
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.contrib.contenttypes.models import ContentType
from django.test import RequestFactory

from djangocms_comments.cms_plugins import CommentsPlugin
from djangocms_comments.models import Comment, CommentsConfig
from djangocms_comments.views import SaveComment
from djangocms_comments.widgets import SignedHiddenInput


class TestBase(object):
    factory = None
    staff_user = None
    normal_user = None
    config = None
    obj = None
    plugin_model_instance = None
    plugin_instance = None

    def setUp(self):
        # Every test needs access to the request factory.
        self.factory = RequestFactory()
        self.anonymous = AnonymousUser()
        self.staff_user = get_user_model().objects.create_superuser(
            username='arthur', email='arthur@nekmo.com', password='grail')
        self.normal_user = get_user_model().objects.create_user(
            username='lancelot', email='lancelot@nekmo.com', password='grail')
        self.config = CommentsConfig.objects.create()
        self.obj = self.normal_user
        self.create_comment(self.obj, self.staff_user, published=True)
        self.create_comment(self.obj, self.normal_user, published=False)
        self.plugin_model_instance, self.plugin_instance = self.create_plugin()

    def create_comment(self, obj, author, published=True):
        Comment.objects.create(page=obj, author=author, body='This is a example comment!', user_ip='127.0.0.1',
                               user_agent='Django Test', published=published, config=self.config)

    def get_page_ct_id(self, obj=None):
        obj = obj or self.obj
        return dict(page_id=obj.pk, page_type=ContentType.objects.get_for_model(obj))

    def get_request(self, user, method='get', data=None, url='/demo', is_test=True, ip=None):
        data = data or {}  # Django 1.6 support
        request = getattr(self.factory, method)(url, data)
        request.user = user or self.anonymous
        request.is_test = is_test
        request.META['REMOTE_ADDR'] = ip or '127.0.0.1'
        return request

    def send_comment(self, author=None, body='', data=None, ajax=True, is_test=True, ip=None, obj=None):
        author = author or self.normal_user
        form = self.render_context_plugin(user=author, obj=obj)['form']
        new_data = {'body': body}
        new_data.update(self.get_sign_values(form))
        new_data.update(data or {})
        request = self.get_request(author, 'post', new_data, '/demo/{0}'.format('?ajax=1' if ajax else ''),
                                   is_test=is_test, ip=ip)
        response = SaveComment.as_view()(request)
        return response

    @staticmethod
    def get_sign_value(form, name):
        return form[name].field.widget.sign_value(name, form[name].value())

    def get_sign_values(self, form=None):
        form = form or self.render_context_plugin()['form']
        data = {}
        for name, bound in form.fields.items():
            if not isinstance(bound.widget, SignedHiddenInput):
                continue
            data[name] = self.get_sign_value(form, name)
        return data

    def render_context_plugin(self, obj=None, request=None, user=None):
        obj = obj or self.obj
        request = request or self.get_request(user or self.normal_user)
        return self.plugin_instance.render({'object': obj, 'request': request}, self.plugin_model_instance, None)

    def create_plugin(self):
        placeholder = Placeholder.objects.create(slot='test')
        model_instance = add_plugin(
            placeholder,
            CommentsPlugin,
            'en',
            config=self.config
        )
        plugin_instance = model_instance.get_plugin_class_instance()
        return model_instance, plugin_instance


import django
from django import forms
from django.forms.fields import ChoiceField
from django.forms.utils import flatatt
from django.forms.widgets import CheckboxChoiceInput, ChoiceFieldRenderer, RadioChoiceInput, SubWidget, RendererMixin, \
    Select, CheckboxInput
from django.utils import html


from django.utils.encoding import python_2_unicode_compatible, force_text
if django.VERSION < (1, 7):
    from django.utils.html import html_safe
else:
    def html_safe(klass):
        return klass
from django.utils.html import format_html


class SubmitButtonWidget(forms.Widget):
    def render(self, name, value, attrs=None):
        return '<input type="submit" name="%s" value="%s">' % (html.escape(name), html.escape(value))


class SubmitButtonField(forms.Field):
    def __init__(self, *args, **kwargs):
        if not kwargs:
            kwargs = {}
        kwargs["widget"] = SubmitButtonWidget

        super(SubmitButtonField, self).__init__(*args, **kwargs)

    def clean(self, value):
        return value


class ButtonGroupRenderer(ChoiceFieldRenderer):
    outer_html = '<div class="btn-group" role="group">{content}</div>'
    inner_html = '{choice_value}{sub_widgets}'


@html_safe
@python_2_unicode_compatible
class ChoiceButton(SubWidget):
    input_type = None  # Subclasses must define this

    def __init__(self, name, value, attrs, choice, index):
        self.choice_value = force_text(choice[0])
        self.choice_label = force_text(choice[1])
        attrs = dict(attrs)
        enabled_class = attrs.pop('enabled_classes', {}).get(self.choice_value, 'btn-primary')
        disabled_class = attrs.pop('dissabled_classes', {}).get(self.choice_value, '')
        self.name = name
        self.value = value
        self.attrs = attrs
        self.index = index
        if 'id' in self.attrs:
            self.attrs['id'] += "_%d" % self.index
        self.attrs['type'] = self.attrs.get('type', 'submit')
        self.attrs['class'] = ' '.join(filter(lambda x: x, [self.attrs.get('class', 'btn'),
                                                            enabled_class if self.value == self.choice_value
                                                            else disabled_class]))

    def __str__(self):
        return self.render()

    def render(self, name=None, value=None, attrs=None, choices=()):
        if self.id_for_label:
            label_for = format_html(' for="{}"', self.id_for_label)
        else:
            label_for = ''
        return format_html(self.tag(attrs))

    def is_checked(self):
        return True

    def tag(self, attrs=None):
        attrs = attrs or self.attrs
        final_attrs = dict(attrs, name=self.name, value=self.choice_value)
        if self.is_checked():
            final_attrs['checked'] = 'checked'
        return format_html('<button{}>{}</button>', flatatt(final_attrs), self.choice_label)

    @property
    def id_for_label(self):
        return self.attrs.get('id', '')


class MultipleSubmitButtonRendered(ButtonGroupRenderer):
    choice_input_class = ChoiceButton


class MultipleSubmitButton(RendererMixin, Select):
    renderer = MultipleSubmitButtonRendered
    _empty_value = ''


class Button(CheckboxInput):
    def __init__(self, attrs=None, check_test=None, values=None):
        self.values = values or ['True', 'False']
        super(Button, self).__init__(attrs, check_test)

    def render(self, name, value, attrs=None):
        attrs = dict(attrs or {})
        attrs.update(self.attrs)
        enabled_class = attrs.pop('enabled_class', 'btn-primary')
        disabled_class = attrs.pop('dissabled_class', '')
        attrs['class'] = ' '.join(filter(lambda x: x, [attrs.get('class', 'btn'),
                                                       enabled_class if value else disabled_class]))
        attrs['type'] = attrs.get('type', 'submit')
        final_attrs = self.build_attrs(attrs, name=name)
        return format_html('<button{}>{}</button>', flatatt(final_attrs), self.values[0 if not value else 1])



import datetime

from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.forms import ModelForm, CharField, EmailField, URLField, IntegerField
from django.forms.widgets import Textarea
from django.utils.translation import ugettext_lazy as _

from djangocms_comments import settings
from djangocms_comments.models import AnonymousAuthor, CommentsConfig
from djangocms_comments.spam import get_spam_protection
from djangocms_comments.widgets import SignedHiddenInput
from .models import Comment


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def is_staff(request):
    return getattr(request.user, 'is_staff', False)


class CommentForm(ModelForm):
    signed_fields = ['config_id', 'page_id', 'page_type']
    config_id = IntegerField(widget=SignedHiddenInput)
    body = CharField(widget=Textarea(attrs={'rows': settings.COMMENT_TEXTAREA_ROWS}), label=_('Your comment'),
                     min_length=settings.MIN_LENGTH_COMMENT_BODY, max_length=settings.MAX_LENGTH_COMMENT_BODY)

    def __init__(self, data=None, **kwargs):
        self.request = kwargs.pop('request', None)
        # data = self.signer_fields(data, 'unsign')
        # kwargs['initial'] = self.signer_fields(kwargs.get('initial', {}))
        super(CommentForm, self).__init__(data, **kwargs)

    def security_validations(self):
        if Comment.objects.filter(user_ip=get_client_ip(self.request),
                                  created_at__gte=datetime.datetime.now() - datetime.timedelta(
                                      seconds=settings.COMMENT_WAIT_SECONDS)).count() \
                and settings.COMMENT_WAIT_SECONDS and not getattr(self.request, 'is_test', False):
            raise ValidationError(_('You must wait to post a new comment.'))

    def clean(self):
        cleaned_data = super(CommentForm, self).clean()
        if self.request and not is_staff(self.request):
            self.security_validations()
        return cleaned_data

    def clean_body(self):
        body = self.cleaned_data['body']
        if not is_staff(self.request):
            if settings.MAX_BREAKLINES_COMMENT_BODY and len(body.split('\n')) > settings.MAX_BREAKLINES_COMMENT_BODY:
                raise ValidationError(_('Your comment has too many line breaks.'))
            if settings.MAX_UPPERCASE_RATIO_COMMENT_BODY and sum(1 for c in body if c.isupper()) / (len(body) * 1.0) > \
                    settings.MAX_UPPERCASE_RATIO_COMMENT_BODY:
                raise ValidationError(_('Your comment has too many uppercase letters.'))
        return body

    def save(self, commit=True, author=None):
        author = author or self.request.user
        user_agent = self.request.META.get('HTTP_USER_AGENT', 'unknown')
        referrer = self.request.META.get('HTTP_REFERER', '')
        user_ip = get_client_ip(self.request)
        published = settings.COMMENT_PUBLISHED_BY_DEFAULT
        is_spam = False
        if is_staff(self.request):
            # Always published for admins
            published = True
        else:
            is_spam = get_spam_protection().check(author, author.email, self.cleaned_data['body'], user_ip, user_agent,
                                                  url=getattr(author, 'website', None), referrer='unknown',
                                                  blog_domain=self.request.get_host())
            published = published and not is_spam
        comment = Comment(author=author, user_ip=user_ip, user_agent=user_agent, referrer=referrer, published=published,
                          config=CommentsConfig.objects.get(pk=self.cleaned_data['config_id']),
                          **dict((key, self.cleaned_data[key]) for key in ['body', 'page_id', 'page_type']))
        if is_spam:
            comment.requires_attention = 'spam'
            comment.moderated = 'spam'
        elif not published:
            comment.requires_attention = 'created'
        if commit:
            comment.save()
        return comment

    class Meta:
        widgets = {
            'page_id': SignedHiddenInput,
            'page_type': SignedHiddenInput,
        }
        model = Comment
        fields = ('body', 'page_type', 'page_id', 'config_id')


class UnregisteredCommentForm(CommentForm):
    username = CharField(max_length=32)
    email = EmailField()
    website = URLField(required=False)

    class Meta:
        widgets = {
            'page_id': SignedHiddenInput,
            'page_type': SignedHiddenInput,
        }
        model = Comment
        fields = ('username', 'email', 'website', 'body', 'page_type', 'page_id', 'config_id')

    def clean_username(self):
        username = self.cleaned_data['username']
        if settings.COMMENT_PREVENT_USURP_USERNAME and get_user_model().objects.filter(username=username).count():
            raise ValidationError(_('The chosen username belongs to a registered user.'))
        return username

    def save(self, commit=True, author=None):
        author, exists = AnonymousAuthor.objects.get_or_create(**dict((key, self.cleaned_data[key]) for key in
                                                                      ['username', 'email', 'website']))
        if not exists:
            author.save()
        return super(UnregisteredCommentForm, self).save(commit=commit, author=author)



import os
from django.test import TestCase

from djangocms_comments.spam import Akismet
from .base import TestBase

USER_AGENT = 'Mozilla/5.0 (X11; Linux x86_64; rv:40.0) Gecko/20100101 Firefox/40.0'
EVIL_USER_AGENT = 'Bot Evil/0.1'


class TestAkismet(TestBase, TestCase):
    def setUp(self):
        super(TestAkismet, self).setUp()
        try:
            akismet_api_key = os.environ['AKISMET_API_KEY']
        except KeyError:
            raise EnvironmentError('Provide AKISMET_API_KEY environment setting.')
        self.akismet = Akismet(akismet_api_key, is_test=True, blog_domain='http://127.0.0.1')

    def test_check_spam(self):
        self.assertTrue(self.akismet.check('viagra-test-123', 'viagra@spam-world.de', 'SPAM SPAM EGGS SPAMMM!!!1',
                                           '127.0.0.1', EVIL_USER_AGENT))

    def test_check_ham(self):
        self.assertFalse(self.akismet.check('Nekmo', 'contacto@nekmo.com', 'Great post! :D',
                                            '127.0.0.1', USER_AGENT))

    def test_report(self):
        self.akismet.report(True, 'Spammer', 'spammer@localhost', 'Great post! :D', '127.0.0.1', EVIL_USER_AGENT)
        self.akismet.report(False, 'Nekmo', 'contacto@nekmo.com', "No spam!", '127.0.0.1', USER_AGENT)



import six

try:
    from django.utils.module_loading import import_string
except ImportError:
    from django.utils.module_loading import import_by_path as import_string

import djangocms_comments
from . import settings

_spam_cache = {}


class SpamProtection(object):
    def __init__(self, token=None, is_test=False, blog_domain=None):
        self.token = token
        self.is_test = is_test
        self.blog_domain = blog_domain

    def check(self, author, email, body, user_ip, user_agent, url=None, referrer='unknown', blog_domain=None):
        raise NotImplementedError

    def report(self, is_spam, author, email, body, user_ip, user_agent, url=None, referrer='unknown', blog_domain=None):
        raise NotImplementedError


class Akismet(SpamProtection):
    user_agent = 'djangocms-comments/{0}'.format(djangocms_comments.__version__)

    def __init__(self, token, is_test=False, blog_domain=None):
        super(Akismet, self).__init__(token, is_test, blog_domain)
        from akismet import Akismet as PyAkismet
        self.akismet = PyAkismet(token, application_user_agent=self.user_agent)

    def get_parameters(self, data):
        return dict(
            blog=self.blog_domain or data['blog_domain'],
            comment_author=data['author'],
            comment_author_email=data['email'],
            comment_author_url=data['url'],
            comment_content=data['body'],
            is_test=self.is_test,
            **dict((key, data[key]) for key in ['user_ip', 'user_agent', 'referrer']))

    def check(self, author, email, body, user_ip, user_agent, url=None, referrer='unknown', blog_domain=None):
        return self.akismet.check(**self.get_parameters(locals()))

    def report(self, is_spam, author, email, body, user_ip, user_agent, url=None, referrer='unknown', blog_domain=None):
        return self.akismet.submit(is_spam, **self.get_parameters(locals()))


class FakeSpamProtection(SpamProtection):
    def check(self, author, email, body, user_ip, user_agent, url=None, referrer='unknown', blog_domain=None):
        return False

    def report(self, is_spam, author, email, body, user_ip, user_agent, url=None, referrer='unknown', blog_domain=None):
        pass


def get_spam_protection(name='default'):
    if name not in _spam_cache:
        spam_protection = settings.SPAM_PROTECTION or {'default': {'BACKEND': FakeSpamProtection}}
        spam_protection = dict(spam_protection)
        conf = spam_protection[name]
        backend = conf.pop('BACKEND')
        conf = dict((key.lower(), value) for key, value in conf.items())
        module = import_string(backend) if isinstance(backend, six.string_types) else backend
        _spam_cache[name] = module(**conf)
    return _spam_cache[name]



from django.conf.urls import *

from djangocms_comments.views import SaveComment

urlpatterns = patterns('',
    # url(r'^$', 'main_view', name='app_main'),
    url(r'^ajax/save_comment$', SaveComment.as_view(), name='djangocms_comments_save_comment'),
)



from django.core.exceptions import SuspiciousOperation
from django.core.signing import Signer, BadSignature
from django.forms import HiddenInput

signer = Signer()


class SignedHiddenInput(HiddenInput):
    def __init__(self, include_field_name=True, attrs=None):
        self.include_field_name = include_field_name
        super(SignedHiddenInput, self).__init__(attrs=attrs)

    def value_from_datadict(self, data, files, name):
        value = super(SignedHiddenInput, self).value_from_datadict(data, files, name)
        try:
            value = signer.unsign(value)
        except BadSignature:
            raise SuspiciousOperation()
        if self.include_field_name:
            name_key = '{0}-'.format(name)
            if not value.startswith(name_key):
                raise SuspiciousOperation()
            value = value.replace(name_key, '', 1)
        return value

    def render(self, name, value, attrs=None):
        value = self.sign_value(name, value)
        return super(SignedHiddenInput, self).render(name, value, attrs=attrs)

    def sign_value(self, name, value):
        if self.include_field_name:
            value = '-'.join(map(str, [name, value]))
        value = signer.sign(value)
        return value

    def value(self):
        pass


import django
from django.contrib import admin
from django.contrib.admin.utils import unquote
from django.db.models import TextField
from django.forms import ModelForm, ChoiceField, CharField, BooleanField
from django.forms.widgets import Input
from django.template.defaultfilters import truncatechars
from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.utils.timesince import timesince
from django.utils.translation import ugettext_lazy as _

from djangocms_comments import settings
from djangocms_comments.fields import SubmitButtonField, MultipleSubmitButtonRendered, MultipleSubmitButton, \
    ButtonGroupRenderer, Button
from djangocms_comments.models import AnonymousAuthor
from djangocms_comments.spam import get_spam_protection, FakeSpamProtection
from .models import CommentsConfig, Comment, MODERATED, REQUIRES_ATTENTION

EMPTY = (None, '---------')
MODERATED = [EMPTY] + MODERATED
REQUIRES_ATTENTION = [EMPTY] + REQUIRES_ATTENTION
CHANGE_TO_CHOICES = [
    ('hidden', _('Hidden')),
    ('published', _('Published')),
    ('spam', _('Spam')),
]

if django.VERSION >= (1, 8):
    # Concat is not available in 1.7 and 1.6
    from django.db.models.functions import Concat


def get_user_agent(agent):
    try:
        from user_agents import parse
    except ImportError:
        return None
    return parse(agent)


class CommentsConfigAdmin(admin.ModelAdmin):
    pass

admin.site.register(CommentsConfig, CommentsConfigAdmin)


class StatusFilter(admin.SimpleListFilter):
    # Human-readable title which will be displayed in the
    # right admin sidebar just above the filter options.
    title = _('Comment status')

    # Parameter for the filter that will be used in the URL query.
    parameter_name = 'status'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """
        return (
            ('published', _('Published')),
            ('hidden', _('Hidden (unpublished)')),
            ('spam', _('Spam')),
            ('edited', _('Edited by admin')),
            ('deleted', _('Deleted')),
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """
        # Compare the requested value (either '80s' or '90s')
        # to decide how to filter the queryset.
        if self.value() == 'published':
            return queryset.filter(moderated='', published=True)
        elif self.value() == 'hidden':
            return queryset.filter(published=False).exclude(moderated='spam')
        elif self.value() == 'spam':
            return queryset.filter(moderated='spam')
        elif self.value() == 'edited':
            return queryset.filter(moderated='edited', published=True)
        elif self.value() == 'deleted':
            return queryset.filter(moderated='deleted', published=True)


class ReadFilter(admin.SimpleListFilter):
    # Human-readable title which will be displayed in the
    # right admin sidebar just above the filter options.
    title = _('Revised comment')

    # Parameter for the filter that will be used in the URL query.
    parameter_name = 'read'

    def lookups(self, request, model_admin):
        """
        Returns a list of tuples. The first element in each
        tuple is the coded value for the option that will
        appear in the URL query. The second element is the
        human-readable name for the option that will appear
        in the right sidebar.
        """
        return (
            (True, _('Yes (read)')),
            (False, _('No (unread)')),
        )

    def queryset(self, request, queryset):
        """
        Returns the filtered queryset based on the value
        provided in the query string and retrievable via
        `self.value()`.
        """
        # Compare the requested value (either '80s' or '90s')
        # to decide how to filter the queryset.
        if self.value() == 'True':
            return queryset.filter(requires_attention='')
        elif self.value() == 'False':
            return queryset.exclude(requires_attention='')


class CommentAdminForm(ModelForm):
    moderated = ChoiceField(choices=MODERATED, required=False)
    requires_attention = ChoiceField(choices=REQUIRES_ATTENTION, required=False)
    # change_to = MultipleSubmitButtonRendered('change_to', 'published', {'enabled_classes': {
    #     'hidden': 'btn-primary', 'spam': 'btn-danger', 'published': 'btn-success'
    # }}, choices=CHANGE_TO_CHOICES)
    change_to = ChoiceField(widget=MultipleSubmitButton(attrs={'enabled_classes': {
        'hidden': 'btn-primary', 'spam': 'btn-danger', 'published': 'btn-success'
    }}), choices=CHANGE_TO_CHOICES, required=False)

    def __init__(self, *args, **kwargs):
        instance = kwargs['instance']
        kwargs['initial'] = kwargs.get('initial', {})
        if instance.moderated == 'spam':
            kwargs['initial']['change_to'] = 'spam'
        else:
            kwargs['initial']['change_to'] = 'published' if instance.published else 'hidden'
        super(CommentAdminForm, self).__init__(*args, **kwargs)

    class Meta:
        model = Comment
        exclude = ()


class CommentAdmin(admin.ModelAdmin):
    change_form_template = 'djangocms_comments/comment-admin-info.html'
    list_filter = (StatusFilter, ReadFilter)
    form = CommentAdminForm
    list_display = ('body_print', 'status', 'author_print', 'page_print', 'created_at_print')
    _user_message = None

    fieldsets = (
        (None, {
            'fields': ('body', 'moderated_reason')
        }),
        ('Advanced options', {
            'classes': ('collapse',),
            'fields': (('moderated', 'requires_attention'), 'config', ('page_type', 'page_id'),
                       ('author_type', 'author_id')),
        }),
    )

    def get_actions(self, request):
        actions = super(CommentAdmin, self).get_actions(request)
        for action in ['make_spam', 'make_published', 'make_soft_deleted', 'make_hidden']:
            actions[action] = (getattr(self, action), action, getattr(getattr(self, action), 'verbose_name', action))
        return actions

    def changeform_view(self, request, object_id=None, form_url='', extra_context=None):
        extra_context = extra_context or {}
        comment = self.get_object(request, unquote(object_id))
        if comment is not None:
            extra_context['user_agent'] = get_user_agent(comment.user_agent) or truncatechars(comment.user_agent, 60)
            comment.requires_attention = ''
            comment.save()
        changeform = super(CommentAdmin, self).changeform_view(request, object_id=object_id, form_url=form_url,
                                                               extra_context=extra_context)
        return changeform

    def save_model(self, request, obj, form, change):
        if obj.pk and form.cleaned_data['body'] != self.model.objects.get(pk=obj.pk).body:
            self.message_user(request, _('The comment has been marked as edited.'))
            obj.moderated_by = request.user
            obj.moderated = 'edited'
        if form.cleaned_data['change_to']:
            obj.moderated_by = request.user
            change_to = form.cleaned_data['change_to']
            if change_to == 'spam':
                obj.moderated = ''
                get_spam_protection().report(True, obj.author, obj.author.email, obj.body, obj.user_ip,
                                             obj.user_agent, url=getattr(obj.author, 'website', None),
                                             referrer=obj.referrer, blog_domain=request.get_host())
                obj.moderated = 'spam'
            obj.published = change_to == 'published'
            msg = {
                'spam': _('The comment has been marked as spam and is no longer visible.'),
                'hidden': _('The comment has been marked as hidden and is no longer visible.'),
                'published': _('The comment has been marked as published.'),
            }[change_to]
            if change_to == 'spam' and get_spam_protection() is not FakeSpamProtection:
                msg += _(' It has also been reported as spam to {0}.').format(get_spam_protection().__class__.__name__)
            self.message_user(request, msg)
        if request.POST.get('toggle_soft_delete'):
            obj.moderated_by = request.user
            obj.moderated = '' if obj.moderated == 'deleted' else 'deleted'
            msg = _('The comment has been moderate and displayed as deleted.') if obj.moderated == 'deleted' \
                else _('The comment has been restored.')
            self.message_user(request, msg)
        return super(CommentAdmin, self).save_model(request, obj, form, change)

    def requires_attention_print(self, obj):
        pass
    requires_attention_print.short_description = _('Unread')

    def body_print(self, obj):
        return self._strong_unread(obj, truncatechars(obj.body, 80))
    body_print.short_description = _('Comment')

    def author_print(self, obj):
        return self._strong_unread(obj, obj.author)
    author_print.short_description = _('Author')
    author_print.admin_order_field = 'author_sort' if django.VERSION >= (1, 8) else None

    def page_print(self, obj):
        return self._strong_unread(obj, obj.page)
    page_print.short_description = _('Page')
    page_print.admin_order_field = 'page_sort' if django.VERSION >= (1, 8) else None

    def created_at_print(self, obj):
        return self._strong_unread(obj, timesince(obj.created_at))
    created_at_print.short_description = _('Created')
    created_at_print.admin_order_field = 'created_at'

    @staticmethod
    def _strong_unread(obj, output):
        if not obj.requires_attention:
            return output
        return mark_safe('<b>{}</b>'.format(escape(output)))

    def status(self, obj):
        status = obj.moderated
        if not obj.published and status != 'spam':
            status = 'hidden'
        elif not status and obj.published:
            status = 'published'
        return mark_safe('<span class="label label-{1}">{0}</span>'.format(
            status, {'spam': 'danger', 'edited': 'info',
                     'published': 'primary', 'deleted': 'warning'}.get(status, 'default')
        ))

    def has_add_permission(self, request):
        return False

    def get_queryset(self, request):
        qs = super(CommentAdmin, self).get_queryset(request)
        if django.VERSION >= (1, 8):
            # Concat is not available in 1.7 and 1.6
            qs = qs.annotate(author_sort=Concat('author_type', 'author_id', output_field=TextField()))
            qs = qs.annotate(page_sort=Concat('page_type', 'page_id', output_field=TextField()))
        return qs

    def _make_base(self, request, queryset, action, message):
        queryset = queryset.exclude(moderated='spam')
        for comment in queryset.all():
            action(comment)
            comment.save()
        message_bit = _("1 comment was") if queryset.count() == 1 else _("{} comment were").format(queryset.count())
        self.message_user(request, "%s successfully reported and marked as spam." % message_bit)

    def make_spam(self, request, queryset):
        def action(comment):
            comment.moderated = 'spam'
        self._make_base(request, queryset.exclude(moderated='spam'), action,
                        _("%s successfully reported and marked as spam."))
    make_spam.verbose_name = _('Mark and report spam')

    def make_soft_deleted(self, request, queryset):
        def action(comment):
            comment.moderated = 'deleted'
            comment.published = True
        self._make_base(request, queryset.exclude(moderated='deleted', published=True), action,
                        _("%s successfully removed usign soft delete."))
    make_soft_deleted.verbose_name = _('Remove usign soft delete')

    def make_published(self, request, queryset):
        def action(comment):
            comment.moderated = ''
            comment.published = True
        self._make_base(request, queryset.exclude(moderated='', published=True), action,
                        _("%s successfully published."))
    make_published.verbose_name = _('Publish')

    def make_hidden(self, request, queryset):
        def action(comment):
            comment.published = False
        self._make_base(request, queryset.exclude(published=False), action,
                        _("%s successfully hidden."))
    make_hidden.verbose_name = _('Hide')

    class Media:
        css = {
            'all': [
                'djangocms_comments/{0}/css/admin-{1}-bootstrap.css'.format(
                    'src' if settings.COMMENTS_FORCE_STATIC_SRC else 'dist',
                    'with' if settings.COMMENTS_ADMIN_WITH_BOOTSTRAP3 else 'without'),
            ],
        }


admin.site.register(Comment, CommentAdmin)


class AnonymousAuthorAdmin(admin.ModelAdmin):
    pass

admin.site.register(AnonymousAuthor, AnonymousAuthorAdmin)



from django.template.loader_tags import register


@register.inclusion_tag('djangocms_comments/admin_submit_line.html', takes_context=True)
def submit_row(context):
    """
    Displays the row of buttons for delete and save.
    """
    from django.contrib.admin.templatetags.admin_modify import submit_row
    row = submit_row(context)
    row['is_soft_deleted'] = context['original'].moderated == 'deleted'
    return row




from django.conf import settings

COMMENT_PUBLISHED_BY_DEFAULT = True
SPAM_PROTECTION = {}  # {'default': {'BACKEND': 'djangocms_comments.spam.Akismet', 'TOKEN': '...'}}
MIN_LENGTH_COMMENT_BODY = 50
MAX_LENGTH_COMMENT_BODY = 12000
MAX_BREAKLINES_COMMENT_BODY = 12
MAX_UPPERCASE_RATIO_COMMENT_BODY = 0.3
COMMENT_TEXTAREA_ROWS = 4
COMMENT_WAIT_SECONDS = 120
COMMENT_PREVENT_USURP_USERNAME = True
COMMENTS_ADMIN_WITH_BOOTSTRAP3 = True
COMMENTS_FORCE_STATIC_SRC = False

BOOTSTRAP3_COLS = 24
BOOTSTRAP3_AVATAR_COLS = 3
BOOTSTRAP3_SELF_ANON_AVATAR_COLS = 5

# Override my settings usign Django Settings
for var_name, value in dict(locals()).items():
    if var_name.isupper():
        locals()[var_name] = getattr(settings, var_name, value)



import math

import cms
import django
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import SuspiciousOperation
from django.test import TestCase

from djangocms_comments import settings
from djangocms_comments.forms import UnregisteredCommentForm
from djangocms_comments.models import Comment
from djangocms_comments.views import SaveComment
from .base import TestBase

PATSY_BODY = """Sir Lancelot: Look, my liege!
King Arthur: [awed] Camelot!
Sir Galahad: Camelot!
Sir Lancelot: Camelot!
"Patsy: It's only a model."""


class CommentsListFilter(TestBase, TestCase):
    def test_context_has_form(self):
        self.assertIn('form', self.render_context_plugin())

    def test_plugin_comment_form(self):
        form = self.render_context_plugin(user=self.anonymous)['form']
        self.assertIsInstance(form, UnregisteredCommentForm)
        self.assertIn('config_id', form.initial)
        self.assertEqual(form.initial['config_id'], self.config.pk)
        self.assertIn('page_type', form.initial)
        self.assertEqual(form.initial['page_type'], ContentType.objects.get_for_model(self.obj).pk)
        self.assertIn('page_id', form.initial)
        self.assertEqual(form.initial['page_id'], self.obj.pk)

    def test_anonymous_comment(self):
        data = {
            'username': 'Patsy', 'email': 'squire@camelot.com', 'website': 'http://camelot.com/test_anonymous_comment',
            'body': PATSY_BODY + ' test_anonymous_comment',
        }
        response = self.send_comment(self.anonymous, data=data)
        self.assertFalse(response.context_data['form'].errors)
        self.assertFalse(response.context_data['form'].is_bound)
        q = dict(config=self.config, **self.get_page_ct_id())
        if django.VERSION < (1, 7):
            q['body'] = data['body']
        else:
            q['anonymous_authors__website'] = data['website']
        self.assertEqual(Comment.objects.filter(**q).count(), 1)

    def test_user_comment(self):
        body = 'CommentsListFilter.test_user_comment test.' + ('This is a test!' * 8)
        response = self.send_comment(self.normal_user, body)
        self.assertFalse(response.context_data['form'].errors)
        self.assertFalse(response.context_data['form'].is_bound)
        try:
            Comment.objects.get(config=self.config, body=body, **self.get_page_ct_id())
        except Comment.DoesNotExist:
            self.fail('The comment has not been created')

    def test_body_len_validation(self):
        # Min
        body = 'spam eggs '
        body = (body * (len(body) // settings.MIN_LENGTH_COMMENT_BODY))[:-1]
        response = self.send_comment(body=body)
        self.assertEqual(len(response.context_data['form'].errors), 1)
        # Max
        body = 'NOBODY expects the Spanish Inquisition! '
        body = (body * (len(body) // settings.MAX_LENGTH_COMMENT_BODY)) + '!'
        response = self.send_comment(body=body)
        self.assertEqual(len(response.context_data['form'].errors), 1)

    def test_max_uppercase_ratio_comment_body(self):
        ratio = settings.MAX_UPPERCASE_RATIO_COMMENT_BODY
        body = 'spam eggs ' * 100
        body = body[:int(math.ceil(len(body) * ratio) + 1)].upper() + body[int(math.ceil(len(body) + 1 * ratio)):]
        response = self.send_comment(body=body)
        self.assertEqual(len(response.context_data['form'].errors), 1)

    def test_max_breaklines_comment_body(self):
        ratio = settings.MAX_BREAKLINES_COMMENT_BODY
        body = '\n'.join(['spam'] * (ratio + 1))
        response = self.send_comment(body=body)
        self.assertEqual(len(response.context_data['form'].errors), 1)

    def test_anonymous_form_error(self):
        body = 'test_anonymous_form_error ' * 100
        response = self.send_comment(self.anonymous, body=body)
        self.assertEqual(len(response.context_data['form'].errors), 2)  # Username, email
        with self.assertRaises(Comment.DoesNotExist):
            Comment.objects.get(config=self.config, body=body,**self.get_page_ct_id())

    def test_user_form_error(self):
        body = ''  # Empty comment is not allowed
        response = self.send_comment(self.normal_user, body=body)
        self.assertEqual(len(response.context_data['form'].errors), 1)  # Comment is required
        with self.assertRaises(Comment.DoesNotExist):
            Comment.objects.get(config=self.config, body=body, **self.get_page_ct_id())

    def test_sign_validation(self):
        signed_values = self.get_sign_values(self.render_context_plugin()['form'])
        for field_name in ['page_id', 'page_type', 'config_id']:
            with self.assertRaises(SuspiciousOperation):
                self.send_comment(data={field_name: signed_values[field_name].split(':')[0]})
            with self.assertRaises(SuspiciousOperation):
                self.send_comment(data={field_name: signed_values[field_name] + '0'})
        with self.assertRaises(SuspiciousOperation):
            self.send_comment(data={'page_id': signed_values['config_id'].split(':')[0]})

    def test_comment_wait_seconds(self):
        author = self.normal_user
        ip = '192.168.1.1'
        self.send_comment(author, ip=ip, is_test=False,
                          body="Listen. Strange women lying in ponds distributing swords is no basis "
                               "for a system of government. Supreme executive power derives from a "
                               "mandate from the masses, not from some farcical aquatic ceremony.")
        new_data = {'body': "You can't expect to wield supreme power just"
                            "'cause some watery tart threw a sword at you! " * 4}
        new_data.update(self.get_sign_values())
        request = self.get_request(author, 'post', new_data, '/demo/?ajax=1', is_test=False, ip=ip)
        response = SaveComment.as_view()(request)
        self.assertEqual(len(response.context_data['form'].errors), 1)

    def test_comment_in_page(self):
        body = 'CommentsListFilter.test_comment_in_page test.' + ('This is a test!' * 8)
        page = cms.api.create_page('Test', 'fullwidth.html', 'en')
        response = self.send_comment(self.normal_user, body, obj=page)
        self.assertEqual(len(response.context_data['form'].errors), 0)
        try:
            Comment.objects.get(config=self.config, body=body)
        except Comment.DoesNotExist:
            self.fail('The comment has not been created')



import django
from django.conf import settings

try:
    from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey, GenericRelation
from django.contrib.contenttypes.models import ContentType
from django.db import models
from cms.models.pluginmodel import CMSPlugin
from django.template.defaultfilters import truncatechars
from django.utils.translation import ugettext_lazy as _

MODERATED = [
    ('spam', _('Spam')),
    ('edited', _('Edited')),
    ('deleted', _('Deleted')),
]

REQUIRES_ATTENTION = [
    ('spam', _('Spam')),
    ('edited', _('Edited')),
    ('created', _('Created')),
]


class CommentsConfig(models.Model):
    name = models.CharField(max_length=32)
    open_comments = models.BooleanField(default=True)
    published_by_default = models.BooleanField(default=True)

    akismet = models.CharField(max_length=12, blank=True)

    def __str__(self):
        return self.name


class CommentsCMSPlugin(CMSPlugin):
    """AppHookConfig aware abstract CMSPlugin class for Aldryn Newsblog"""
    # avoid reverse relation name clashes by not adding a related_name
    # to the parent plugin

    config = models.ForeignKey(CommentsConfig)

    class Meta:
        abstract = True

    def copy_relations(self, old_instance):
        self.config = old_instance.config


class Comment(models.Model):
    config = models.ForeignKey(CommentsConfig)

    page_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, related_name='+')
    page_id = models.PositiveIntegerField()
    page = GenericForeignKey('page_type', 'page_id')

    author_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, related_name='+')
    author_id = models.PositiveIntegerField()
    author = GenericForeignKey('author_type', 'author_id')

    body = models.TextField()

    requires_attention = models.CharField(blank='', max_length=16, choices=REQUIRES_ATTENTION)
    moderated = models.CharField(blank='', max_length=16, choices=MODERATED)
    moderated_reason = models.CharField(max_length=120, blank=True)
    moderated_by = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)

    user_ip = models.GenericIPAddressField()
    # http://stackoverflow.com/questions/654921/how-big-can-a-user-agent-string-get
    user_agent = models.TextField()
    referrer = models.URLField(blank=True)
    published = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    @property
    def is_edited(self):
        return self.updated_at.replace(second=0, microsecond=0) > self.created_at.replace(second=0, microsecond=0)

    def __str__(self):
        return u'[{}] {}'.format(self.page, truncatechars(self.body, 40))

    def __unicode__(self):
        return self.__str__()


class AnonymousAuthor(models.Model):
    username = models.CharField(max_length=32)
    email = models.EmailField()
    website = models.URLField(blank=True)
    comments = GenericRelation(Comment, object_id_field='author_id', content_type_field='author_type',
                               **{('related_name' if django.VERSION < (1, 7)
                                   else 'related_query_name'): 'anonymous_authors'})

    def __str__(self):
        return self.username

    def __unicode__(self):
        return self.username


class Comments(CommentsCMSPlugin):
    published_by_default = models.BooleanField(default=True)



from django.contrib.auth.models import AnonymousUser
from django.test import TestCase

from djangocms_comments.cms_plugins import CommentsPlugin
from djangocms_comments.models import Comment
from .base import TestBase


class CommentsListFilter(TestBase, TestCase):

    def test_anonymous_only_published(self):
        request = self.factory.get('/demo')
        request.user = AnonymousUser()
        self.assertEquals(CommentsPlugin.get_comments(request, self.obj).count(), Comment.objects.filter(
            config=self.config, published=True, **self.get_page_ct_id()
        ).count())

    def test_normal_user_only_published(self, comments=None):
        request = self.factory.get('/demo')
        request.user = self.normal_user
        comments = comments or CommentsPlugin.get_comments(request, self.obj)
        self.assertEquals(comments.count(), Comment.objects.filter(
            config=self.config, published=True, **self.get_page_ct_id()
        ).count())

    def test_staff_all_comments(self):
        request = self.factory.get('/demo')
        request.user = self.staff_user
        self.assertEquals(CommentsPlugin.get_comments(request, self.obj).count(), Comment.objects.filter(
            config=self.config, **self.get_page_ct_id()
        ).count())

    def test_plugin_context(self):
        request = self.factory.get('/demo')
        request.user = self.normal_user
        context = self.plugin_instance.render({'object': self.obj, 'request': request}, self.plugin_model_instance,
                                              None)
        self.assertIn('comments', context)
        self.test_normal_user_only_published(context['comments'])
        self.assertIn('instance', context)
        self.assertEquals(context['instance'], self.plugin_model_instance)



import six
from django import template
import hashlib

register = template.Library()


# {{ "some identifier"|md5 }} -> g87g98ht02497hg349ugh3409h34
@register.filter(name='md5')
def md5_string(value):
    value = six.b(value)
    return hashlib.md5(value).hexdigest()


@register.filter('substract')
def subtract(value, arg):
    return value - arg



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
        ('cms', '0013_urlconfrevision'),
    ]

    operations = [
        migrations.CreateModel(
            name='AnonymousAuthor',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('username', models.CharField(max_length=32)),
                ('email', models.EmailField(max_length=254)),
                ('website', models.URLField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('page_id', models.PositiveIntegerField()),
                ('author_id', models.PositiveIntegerField()),
                ('body', models.TextField()),
                ('requires_attention', models.CharField(choices=[('spam', 'Spam'), ('edited', 'Edited')], max_length=16, blank='')),
                ('moderated', models.CharField(choices=[('spam', 'Spam'), ('edited', 'Edited'), ('deleted', 'Deleted')], max_length=16, blank='')),
                ('moderated_reason', models.CharField(max_length=120, blank=True)),
                ('user_ip', models.GenericIPAddressField()),
                ('user_agent', models.TextField()),
                ('referrer', models.URLField(blank=True)),
                ('published', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('author_type', models.ForeignKey(to='contenttypes.ContentType', related_name='+')),
            ],
        ),
        migrations.CreateModel(
            name='Comments',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(to='cms.CMSPlugin', primary_key=True, parent_link=True, serialize=False, auto_created=True)),
                ('published_by_default', models.BooleanField(default=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='CommentsConfig',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=32)),
                ('open_comments', models.BooleanField(default=True)),
                ('published_by_default', models.BooleanField(default=True)),
                ('akismet', models.CharField(max_length=12, blank=True)),
            ],
        ),
        migrations.AddField(
            model_name='comments',
            name='config',
            field=models.ForeignKey(to='djangocms_comments.CommentsConfig'),
        ),
        migrations.AddField(
            model_name='comment',
            name='config',
            field=models.ForeignKey(to='djangocms_comments.CommentsConfig'),
        ),
        migrations.AddField(
            model_name='comment',
            name='moderated_by',
            field=models.ForeignKey(null=True, to=settings.AUTH_USER_MODEL, blank=True),
        ),
        migrations.AddField(
            model_name='comment',
            name='page_type',
            field=models.ForeignKey(to='contenttypes.ContentType', related_name='+'),
        ),
    ]



from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _

from djangocms_comments import settings
from djangocms_comments.views import get_form_class, get_is_user
from .models import Comments, Comment


def get_object_from_context(context):
    return context.get('object') or context['current_page']


class CommentsPlugin(CMSPluginBase):
    model = Comments
    render_template = "djangocms_comments/comments.html"
    cache = False

    def render(self, context, instance, placeholder):
        obj = get_object_from_context(context)
        request = context['request']
        ct = ContentType.objects.get_for_model(obj)
        context['comments'] = self.get_comments(request, obj, ct)
        context['use_src'] = settings.COMMENTS_FORCE_STATIC_SRC
        context['comments_settings'] = settings
        initial = {
            'config_id': instance.config.pk,
            'page_type': ct.pk,
            'page_id': obj.pk,
        }
        context['form'] = get_form_class(request)(initial=initial)
        context['is_user'] = get_is_user(request)
        context['is_staff'] = request.user.is_staff
        return super(CommentsPlugin, self).render(context, instance, placeholder)

    @staticmethod
    def get_comments(request, obj, ct=None):
        ct = ct or ContentType.objects.get_for_model(obj)
        comments = Comment.objects.filter(page_type=ct, page_id=obj.pk)
        if not getattr(request.user, 'is_staff', False):
            comments = comments.filter(published=True).exclude(moderated='spam')
        return comments


plugin_pool.register_plugin(CommentsPlugin)



from django.shortcuts import redirect, render
from django.utils.translation import ugettext_lazy as _
from django.views.generic.edit import FormView

from djangocms_comments import settings
from djangocms_comments.forms import UnregisteredCommentForm, CommentForm


def get_form_class(request):
    if request.user.is_anonymous():
        return UnregisteredCommentForm
    else:
        return CommentForm


def get_is_user(request):
    return not request.user.is_anonymous()


class SaveComment(FormView):
    comment = None
    template_name = 'djangocms_comments/comment_box.html'

    def get_form_class(self):
        return get_form_class(self.request)

    def get_form(self, form_class=None):
        if form_class is None:
            form_class = self.get_form_class()
        form = form_class(self.request.POST, request=self.request)
        return form

    def form_valid(self, form):
        self.comment = form.save()
        return form

    def form_invalid(self, form):
        return form

    def create_new_form(self, old_form):
        initial = {
            'config_id': old_form.cleaned_data['config_id'],
            'page_type': old_form.cleaned_data['page_type'],
            'page_id': old_form.cleaned_data['page_id'],
        }
        return self.get_form_class()(initial=initial, request=self.request)

    def post(self, request, *args, **kwargs):
        form = super(SaveComment, self).post(request, *args, **kwargs)
        if request.GET.get('ajax'):
            return self.render_to_response({
                'form': self.create_new_form(form) if form.is_valid() else form,
                'comment': self.comment,
                'is_user': get_is_user(request),
                'comments_settings': settings,
            })
        referrer = request.META.get('HTTP_REFERER', '')
        return redirect(referrer)
